---
layout: skeleton
---
{% if page.lang_suffix == null %}
<nav class="menu">
{% include en.menu.html %}
</nav>
{% endif %}
{% if page.no_sidebar %}
  {{ content }}
{% else %}
<div id="body">
  {{ content }}
  <nav id="sidebar">
    <h3>The Latest Release</h3>
    <ul>
      <li>
        Rabbit <a href="/en/news.html">{{ site.version }}</a>:
        {{ site.release_date }}
      </li>
      <li>
        <a href="/en/rabbiter/">Rabbiter</a> <a href="/en/rabbiter/news.html">{{ site.rabbiter_version }}</a>:
        {{ site.rabbiter_release_date }}
      </li>
      <li>
        <a href="/en/rabbirack">RabbiRack</a> <a href="/en/rabbirack/news.html">{{ site.rabbirack_version }}</a>:
        {{ site.rabbirack_release_date }}
      </li>
      <li>
        <a href="/en/rabwii">RabWii</a> <a href="/en/rabwii/news.html">{{ site.rabwii_version }}</a>:
        {{ site.rabwii_release_date }}
      </li>
    </ul>
    <h3>Links</h3>
{% include en.menu.html %}
    <h3>Related sites</h3>
    <ul class="related-sites">
      <li id="rabbit-slide-show">
        <a href="https://slide.rabbit-shocker.org/"
           title="Rabbit Slide Show">
          <img src="/images/rabbit-slide-show-logo.png"
               alt="Rabbit Slide Show">
        </a>
      </li>
    </ul>
  </nav>
</div>
{% endif %}
