<!DOCTYPE html>
<html lang="{{ page.language }}">
  <head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Rabbit - {{ page.title }}</title>
    <link rel="shortcut icon" href="/favicon.ico"
          type="image/vnd.microsfot.icon">
    <link rel="icon" href="/favicon.png" type="image/png">
    <link rel="stylesheet" href="/css/syntax.css" type="text/css">
    <link rel="stylesheet" href="/css/rabbit.css" type="text/css">
    <meta property="fb:page_id" content="rabbit.shocker">
    <meta property="fb:admins" content="kouhei.sutou">
    <meta property="og:type" content="product">
    <meta property="og:image"
          content="http://rabbit-shocker.org/images/lavie.png">
    <meta property="og:site_name" content="Rabbit">
    <script>
      (function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
      (i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
      m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
      })(window,document,'script','//www.google-analytics.com/analytics.js','ga');

      ga('create', 'UA-12143252-4', 'rabbit-shocker.org');
      ga('send', 'pageview');
    </script>
  </head>
  <body>
    <div id="fb-root"></div>
    <script>(function(d, s, id) {
  var js, fjs = d.getElementsByTagName(s)[0];
  if (d.getElementById(id)) return;
  js = d.createElement(s); js.id = id;
  js.src = "//connect.facebook.net/{{ page.language }}_{{ page.country }}/all.js#xfbml=1";
  fjs.parentNode.insertBefore(js, fjs);
}(document, 'script', 'facebook-jssdk'));</script>

    <header id="header">
      <h1 class="title">
        <a href="/">
          <span class="project">Rabbit</span>
          <span class="separator">-</span>
          <span class="description">{{ page.title }}</span>
        </a>
      </h1>
      <div class="facebook-buttons">
        <div class="fb-like"
             data-href="http://www.facebook.com/pages/Rabbit/202603309766491"
             data-layout="standard"
             data-send="true"
             data-show-faces="true"
             data-width="320"></div>
      </div>
      <pre id="install" class="command">gem install rabbit</pre>
      <div class="other-language-links">
        <ul>
{% if page.language != "en" %}
  {% capture en_url %}
    {% if page.lang_suffix %}
      {{ page.url | remove:page.language | append:'en' }}
    {% else %}
      /en{{ page.url | remove_first:page.language | replace:'//','/' | remove:"index.html" }}
    {% endif %}
  {% endcapture %}
          <li><a href="{{ en_url | remove:' ' | strip_newlines }}">English</a></li>
{% endif %}
{% if page.language != "ja" %}
  {% capture ja_url %}
    {% if page.lang_suffix %}
      {{ page.url | remove:page.language | append:'ja' }}
    {% else %}
      /ja{{ page.url | remove_first:page.language | replace:'//','/' | remove:"index.html" }}
    {% endif %}
  {% endcapture %}
          <li><a href="{{ ja_url | remove:' ' | strip_newlines }}">日本語</a></li>
{% endif %}
        </ul>
      </div>
    </header>
    {{ content }}
    <footer>
      <p class="copyright">&copy; 2004-{{ site.time | date:"%Y" }} Kouhei Sutou.</p>
    </footer>
  </body>
</html>
