@category = N_("Toolkit")
@title = N_("DefaultDescription")
@abstract = N_("Toolkit to display description-list items with orange " \
               "underlines.")
@description = N_("Displays description-list items with orange underlines.")
