@category = N_("Toolkit")
@title = N_("DefaultTitleSlide")
@abstract = N_("Toolkit to locate objects in the title slide in a simple way")
@description = N_("Locates objects in the title slide simply by centering " \
                  "them and by making configuration for margins around them " \
                  "a little.")
