@category = N_("Toolkit")
@title = N_("RabbitTitleLogo")
@abstract = N_("Toolkit to display Lavie as a logo in the title slide")
@description = N_("Displays Lavie as a logo in the title slide.")
@dependencies = %w(rabbit-images headline-logo)
