@category = N_("Toolkit")
@title = N_("DefaultIcon")
@abstract = N_("Toolkit to use pink circle and polygon as icons")
@description = N_("Uses pink circle and polygon as icons.")
@dependencies = %w(ruby-images icon)
