@category = N_("Toolkit")
@title = N_("COZMIXNGPoweredBy")
@abstract = N_("Powered-by-COZMIXNG ad toolkit")
@description = N_("Displays a text and images showing the presentation " \
                  "slides are powered by COZMIXNG and Rabbit at the foot of " \
                  "the title slide and of the last slide.")
@dependencies = %w(cozmixng-images rabbit-images powered-by)
