@category = N_("Toolkit")
@title = N_("RubyGNOME2Description")
@abstract = N_("Toolkit to display description-list items like ones in the " \
               "website of the Ruby-GNOME2 Project")
@description = N_("Displays description-list items like ones in the website " \
                  "of the Ruby-GNOME2 Project; i.e. displays their text " \
                  "colored blue.")
