@category = N_("Toolkit")
@title = N_("SimpleItemMark")
@abstract = N_("Toolkit to display simple black circles as list item marks")
@description = N_("Displays simple black circles at the head of list items.")
@dependencies = %w(default-item-mark)
