@category = N_("Backward compatibility")
@title = N_("SlideBackground")
@abstract = N_("Backward compatibility theme for ((<slide-background-image>))")
@description = N_("See ((<slide-background-image>))")
@parameters = {
}
