# NOTE: This is sumple theme

include_theme("rabbit")

@image_timer_limit = 3 * 60 # customize
include_theme("image-timer")

@slide_number_uninstall = true
include_theme("slide-number")

include_theme("image-slide-number")
