= <%= _(@title) %>

: <%= _("Name") %>
   <%= @name %>

: <%= _("Category") %>
   <%= _(category) %>

<% if @abstract %>
: <%= _("Abstract") %>
   <%= _(@abstract) %>
<% end %>

<% if @description %>
== <%= _("Description") %>

<%= _(@description) %>
<% end %>

<% unless @dependencies.empty? %>
== <%= _("Dependency themes") %>
<%   @dependencies.each do |theme| %>
  * ((<<%= theme %>>))
<%   end %>
<% end %>

<% unless @parameters.empty? %>
== <%= _("Parameters") %>
<%   @parameters.each do |name, info| %>
: ((|<%= name %>|))
   (<%= _("Default: ") %>(({<%= _(info[:default]) %>})))

   <%= _(info[:description]) %>
<%   end %>
<% end %>
