@default_foreground = "#000000"
@default_background = "#ffffff"

@default_item1_mark_color = "#a00000"
@default_item1_mark_type = "circle"
@default_item2_mark_color = "#a00000"
@default_item2_mark_type = "circle"
include_theme("default")
