@category = N_("Toolkit")
@title = N_("PerSlideBackgroundImage")
@abstract = N_("Toolkit to display an image as a background of each slide")
@description = N_("Displays an image as a background of each slide.\n" \
                  "\n" \
                  "Each image is specified as a slide property:\n" \
                  "  = target slide\n" \
                  "  \n" \
                  "  ...\n" \
                  "  \n" \
                  "  == properties\n" \
                  "  \n" \
                  "  : background-image\n" \
                  "     my-picture.png\n")
@parameters = {
}
