@category = N_("Toolkit")
@title = N_("PerSlideBackgroundColor")
@abstract = N_("Toolkit to set background color of each slide")
@description = N_("Set background color of each slide.\n" \
                  "\n" \
                  "Each color is specified as a slide property:\n" \
                  "  = target slide\n" \
                  "  \n" \
                  "  ...\n" \
                  "  \n" \
                  "  == properties\n" \
                  "  \n" \
                  "  : background-color\n" \
                  "     black\n")
@parameters = {
}
