# Copyright (C) 2004-2025  Sutou Kouhei <kou@cozmixng.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require_relative "../../gtk"

module Rabbit
  module Renderer
    module Display
      module MotionHandler
        private
        if Gtk::Widget.method_defined?(:add_controller)
          def set_motion_event(widget)
            motion = Gtk::EventControllerMotion.new
            motion.signal_connect(:motion) do |_, x, y|
              call_hook_procs(@motion_notify_hook_procs, x, y)
            end
            widget.add_controller(motion)
          end
        else
          def set_motion_event(widget)
            widget.add_events(Gdk::EventMask::BUTTON1_MOTION_MASK |
                              Gdk::EventMask::BUTTON2_MOTION_MASK |
                              Gdk::EventMask::BUTTON3_MOTION_MASK)
            widget.signal_connect("motion_notify_event") do |widget, event|
              call_hook_procs(@motion_notify_hook_procs, event.x, event.y)
            end
          end
        end
      end
    end
  end
end
