@category = N_("Toolkit")
@title = N_("TitleShadow")
@abstract = N_("Toolkit to display shadows behind the title")
@description = N_("Displays shadows behind the title string in the title " \
                  "slide.")
@parameters = {
  "@title_shadow_color" => {
    :default => "'#6f6f6f'",
    :description => N_("Color of shadow."),
  },
}
