@category = N_("Toolkit")
@title = N_("RubyGNOME2ItemMark")
@abstract = N_("Toolkit to display list items like ones in the website of " \
               "the Ruby-GNOME2 Project")
@description = N_("Displays list items like ones in the website of the " \
                  "Ruby-GNOME2 Project; i.e. displays text of the " \
                  "first-level items colored blue and underlined, the " \
                  "second- and third-level items marked with black squares " \
                  "and circles.")
@dependencies = %w(default-item-mark)
