@category = N_("Toolkit")
@title = N_("RabbitPoweredBy")
@abstract = N_("Powered-by-Rabbit ad toolkit")
@description = N_("Displays a text and an image showing the presentation " \
                  "slides are powered by Rabbit and COZMIXNG at the foot of " \
                  "the title slide and of the last slide.")
@dependencies = %w(rabbit-images powered-by)
