@default_foreground = "#ffffff"
@default_background = "#303030"

@image_caption_color = @default_foreground
@table_caption_color = @default_foreground

@default_item1_mark_color = "#a00000"
@default_item1_mark_type = "circle"
@default_item2_mark_color = "#a00000"
@default_item2_mark_type = "circle"

@preformatted_fill_color = "#101010"

@block_quote_fill_color = "#101030"

@table_head_fill_color = "#101030"
@table_body_fill_color = "#101010"

include_theme("default")
