@color_circle_foreground = "black"
@color_circle_background = "white"

@color_circle_color = "#6fd195"
@color_circle_light_color = "#d2ffe2"
@color_circle_bright_color ="#dfffec"

@color_circle_graffiti_color = "green"

# add_image_path("rabbit-images")
# @color_circle_open_quote_image = "open-quote-green.png"
# @color_circle_close_quote_image = "close-quote-green.png"

include_theme("color-circle")
