<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html
    PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<meta http-equiv="Content-Style-Type" content="text/css" />
<%= rss_link %>
<style type="text/css">
div.page-navi
{
  text-align: center;
}

div.slide,
div.index-slide
{
  margin-top: 1em;
  margin-bottom: 1em;
  margin-left: auto;
  margin-right: auto;
}

div.slide
{
  text-align: center;
}

div.index-slide
{
  width: 90%;
}

div.slide img
{
  border-style: none;
}
</style>
<script type="text/javascript">
function handle_key(e)
{
  var key_code = e.which || e.keyCode;
  var handled = true;
  switch (key_code) {

<% unless first_slide? -%>
<% Rabbit::Keys::MOVE_TO_FIRST_KEYS.each do |key| -%>
  case <%= key %>:
<% end -%>
    location = "<%= first_href %>";
    break;

<% Rabbit::Keys::MOVE_TO_PREVIOUS_KEYS.each do |key| -%>
  case <%= key %>:
<% end -%>
    location = "<%= previous_href %>";
    break;
<% end -%>

<% unless last_slide? -%>
<% Rabbit::Keys::MOVE_TO_NEXT_KEYS.each do |key| -%>
  case <%= key %>:
<% end -%>
    location = "<%= next_href %>";
    break;

<% Rabbit::Keys::MOVE_TO_LAST_KEYS.each do |key| -%>
  case <%= key %>:
<% end -%>
    location = "<%= last_href %>";
    break;
<% end -%>

<% if (!outputting_index? and output_index_html?) or
        (outputting_index? and output_html?) -%>
<% Rabbit::Keys::TOGGLE_INDEX_MODE_KEYS.each do |key| -%>
  case <%= key %>:
<% end -%>
    location = "<%= toggle_mode_href %>";
    break;
<% end -%>

  default:
    handled = false;
    break;
  }

  return !handled;
}
</script>
<title><%=h slide_title %></title>
</head>
<body onkeypress="handle_key(event);">
<%= navi %>
<%= page_navi %>
<%= slide_content %>
<%= page_navi %>
<%= navi %>
<hr />
<div class="footer">
<p>
Generated by
<a href="http://rabbit-shocker.org/">Rabbit</a>
version <%= Rabbit::VERSION %>
</p>
</div>
</body>
</html>
