@category = N_("Toolkit")
@title = N_("TitleBackgroundColor")
@abstract = N_("Toolkit to color a background of the title slide")
@description = N_("Color a background of the title slide.")
@parameters = {
  "@title_background_color" => {
    :default => N_("(Must be specified.)"),
    :description => N_("Background color.")
  },
}
