# Copyright (C) 2017-2025  Sutou Kouhei <kou@cozmixng.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

require_relative "element/block"
require_relative "element/description-list"
require_relative "element/enum-list"
require_relative "element/footnote"
require_relative "element/image-title-slide"
require_relative "element/image"
require_relative "element/index-slide"
require_relative "element/item-list"
require_relative "element/method-list"
require_relative "element/poppler-slide"
require_relative "element/poppler-title-slide"
require_relative "element/preformatted"
require_relative "element/slide"
require_relative "element/syntax-highlighting"
require_relative "element/table"
require_relative "element/tag"
require_relative "element/text"
require_relative "element/title-slide"
require_relative "element/video"
