@category = N_("Toolkit")
@title = N_("RubyGNOME2Headline")
@abstract = N_("Toolkit to display headlines like those in the website of " \
               "the Ruby-GNOME2 Project")
@description = N_("Displays headlines like ones in the website of the " \
                  "Ruby-GNOME2 Project; i.e. displays their text colored " \
                  "white and placed in the dark red boxes with pink frames.")
