@category = N_("Toolkit")
@title = N_("TitleBackgroundImage")
@abstract = N_("Toolkit to display an image as a background of the title " \
               "slide")
@description = N_("Displays an image as a background of the title slide.")
@parameters = {
  "@title_background_image" => {
    :default => N_("(Must be specified.)"),
    :description => N_("Image file name.")
  },
}
