@category = N_("Toolkit")
@title = N_("Ruby-GNOME2 Foot Text")
@description = N_("Displays foot text at the footer of slide " \
                  "with Ruby-GNOME2 style.")
@dependencies = %w(default-foot-text)
